<?php

namespace Drupal\wowjs_ui\Form;

use Drupal\animatecss_ui\Form\AnimateCssForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * The WOW JS add and edit wow alter on animatecss form.
 *
 * @internal
 */
class WowJsForm extends AnimateCssForm {

  /**
   * An array to store the variables.
   *
   * @var array
   */
  protected $variables = [];

  /**
   * {@inheritdoc}
   *
   * @param array $form
   *   A nested array form elements comprising the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param int $animate
   *   (optional) WOW id to be passed on to
   *   \Drupal::formBuilder()->getForm() for use as the default value of the
   *   WOW ID form data.
   */
  public function buildForm(array $form, FormStateInterface $form_state, int $animate = 0) {
    // Retrieve the original form.
    $form = parent::buildForm($form, $form_state, $animate);

    $animate = is_array($this->animateManager->findById($animate)) ? $this->animateManager->findById($animate) : [];
    $options = unserialize($animate['options'], ['allowed_classes' => FALSE]) ?? '';

    // Load the WOW JS configuration settings.
    $config = $this->config('wowjs.settings');

    // WOW JS options.
    $form['wow_options'] = [
      '#title' => $this->t('WOW options'),
      '#type'  => 'details',
      '#open'  => TRUE,
    ];

    // Enable WOW to reveal CSS animation as you scroll down a page.
    $form['wow_options']['wow'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Enable WOW'),
      '#description'   => $this->t("Reveal CSS animation as you scroll down a page."),
      '#default_value' => $options['wow']['enable'] ?? FALSE,
    ];

    $form['wow_options']['data'] = [
      '#type' => 'container',
      '#states' => [
        'disabled' => [
          ':input[name="wow"]' => ['checked' => FALSE],
        ],
        'visible' => [
          ':input[name="wow"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Offset WOW animation.
    $form['class_box'] = [
      '#type'  => 'value',
      '#value' => $config->get('options.classBox'),
    ];

    // Offset WOW animation.
    $form['wow_options']['data']['offset'] = [
      '#type'          => 'number',
      '#min'           => 0,
      '#title'         => $this->t('Offset'),
      '#default_value' => $options['wow']['offset'] ?? $config->get('options.offset'),
      '#field_suffix'  => 'px',
    ];

    // Trigger animations on mobile devices (default is true).
    $form['wow_options']['data']['mobile'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Mobile'),
      '#description'   => $this->t("Trigger animations on mobile devices."),
      '#default_value' => $options['wow']['mobile'] ?? $config->get('options.mobile'),
    ];

    // Act on asynchronously loaded content (default is true).
    $form['wow_options']['data']['live'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Live'),
      '#description'   => $this->t("Act on asynchronously loaded content."),
      '#default_value' => $options['wow']['live'] ?? $config->get('options.live'),
    ];

    // Enable scroll container selector, otherwise use window.
    $form['wow_options']['data']['optional_container'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Scroll container'),
      '#description'   => $this->t('Optional scroll container selector, otherwise use window.'),
      '#default_value' => $options['wow']['optionalContainer'] ?? $config->get('options.optionalContainer'),
    ];

    // Optional scroll container selector.
    $form['wow_options']['data']['scroll_container'] = [
      '#type'          => 'textfield',
      '#title_display' => 'invisible',
      '#title'         => $this->t('Container selector'),
      '#description'   => $this->t('Scroll container selector.'),
      '#default_value' => $options['wow']['scrollContainer'] ?? $config->get('options.scrollContainer'),
      '#states'        => [
        'visible' => [
          ':input[name="optional_container"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    // The WOW main variables.
    $variables['wow']['enable'] = $values['wow'];
    $variables['wow']['offset'] = $values['offset'];
    $variables['wow']['mobile'] = $values['mobile'];
    $variables['wow']['live']   = $values['live'];

    // The WOW advanced wow settings.
    $variables['wow']['optionalContainer'] = $values['optional_container'];
    $variables['wow']['scrollContainer']   = $values['scroll_container'];

    // Set wow variables.
    $this->variables = $variables;

    // Now submit AnimateCSS form.
    parent::submitForm($form, $form_state);
  }

  /**
   * Submit handler for removing wow.
   *
   * @param array[] $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function remove(&$form, FormStateInterface $form_state) {
    $wid = $form_state->getValue('wow_id');
    $form_state->setRedirect('wowjs.delete', ['wid' => $wid]);
  }

  /**
   * Form submission handler for the 'overview' action.
   *
   * @param array[] $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function overview(array $form, FormStateInterface $form_state): void {
    $form_state->setRedirect('animatecss.admin');
  }

}
